@@ -1,6 +1,6 @@
 /*
    AngelCode Scripting Library
-   Copyright (c) 2003-2013 Andreas Jonsson
+   Copyright (c) 2003-2014 Andreas Jonsson
 
    This software is provided 'as-is', without any express or implied
    warranty. In no event will the authors be held liable for any
@@ -34,6 +34,8 @@
 //
 // These functions handle the actual calling of system functions
 //
+// Added support for functor methods by Jordi Oliveras Rovira in April, 2014.
+//
 
 
 
@@ -87,7 +89,7 @@ asQWORD CallThisCallFunctionRetByRef(const void *, const asDWORD *, int, asFUNCT
 asDWORD GetReturnedFloat();
 asQWORD GetReturnedDouble();
 
-asQWORD CallSystemFunctionNative(asCContext *context, asCScriptFunction *descr, void *obj, asDWORD *args, void *retPointer, asQWORD &/*retQW2*/)
+asQWORD CallSystemFunctionNative(asCContext *context, asCScriptFunction *descr, void *obj, asDWORD *args, void *retPointer, asQWORD &/*retQW2*/, void *secondObject)
 {
 	asCScriptEngine            *engine    = context->m_engine;
 	asSSystemFunctionInterface *sysFunc   = descr->sysFuncIntf;
@@ -95,13 +97,30 @@ asQWORD CallSystemFunctionNative(asCContext *context, asCScriptFunction *descr,
 	asQWORD retQW = 0;
 
 	// Prepare the parameters
-	int paramSize = sysFunc->paramSize;
 	asDWORD paramBuffer[64];
-	if( sysFunc->takesObjByVal )
+	int callConv = sysFunc->callConv;
+
+	// Changed because need check for ICC_THISCALL_OBJFIRST or
+	// ICC_THISCALL_OBJLAST if sysFunc->takesObjByVal (avoid copy code)
+	// Check if is THISCALL_OBJ* calling convention (in this case needs to add secondObject pointer into stack).
+	bool isThisCallMethod = callConv >= ICC_THISCALL_OBJLAST;
+	int paramSize = isThisCallMethod || sysFunc->takesObjByVal ? 0 : sysFunc->paramSize;
+
+	int dpos = 1;
+
+	if( isThisCallMethod && 
+		(callConv >= ICC_THISCALL_OBJFIRST &&
+		 callConv <= ICC_VIRTUAL_THISCALL_OBJFIRST_RETURNINMEM) )
+	{
+		// Add the object pointer as the first parameter
+		paramBuffer[dpos++] = (asDWORD)secondObject;
+		paramSize++;
+	}
+
+	if( sysFunc->takesObjByVal || isThisCallMethod )
 	{
-		paramSize = 0;
 		int spos = 0;
-		int dpos = 1;
+
 		for( asUINT n = 0; n < descr->parameterTypes.GetLength(); n++ )
 		{
 			if( descr->parameterTypes[n].IsObject() && !descr->parameterTypes[n].IsObjectHandle() && !descr->parameterTypes[n].IsReference() )
@@ -146,9 +165,17 @@ asQWORD CallSystemFunctionNative(asCContext *context, asCScriptFunction *descr,
 		args = &paramBuffer[1];
 	}
 
+	if( isThisCallMethod && 
+		(callConv >= ICC_THISCALL_OBJLAST &&
+		 callConv <= ICC_VIRTUAL_THISCALL_OBJLAST_RETURNINMEM) )
+	{
+		// Add the object pointer as the last parameter
+		paramBuffer[dpos++] = (asDWORD)secondObject;
+		paramSize++;
+	}
+
 	// Make the actual call
 	asFUNCTION_t func = sysFunc->func;
-	int callConv = sysFunc->callConv;
 	if( sysFunc->hostReturnInMemory )
 		callConv++;
 
@@ -176,14 +203,20 @@ asQWORD CallSystemFunctionNative(asCContext *context, asCScriptFunction *descr,
 		break;
 
 	case ICC_THISCALL:
+	case ICC_THISCALL_OBJFIRST:
+	case ICC_THISCALL_OBJLAST:
 		retQW = CallThisCallFunction(obj, args, paramSize<<2, func);
 		break;
 
 	case ICC_THISCALL_RETURNINMEM:
+	case ICC_THISCALL_OBJFIRST_RETURNINMEM:
+	case ICC_THISCALL_OBJLAST_RETURNINMEM:
 		retQW = CallThisCallFunctionRetByRef(obj, args, paramSize<<2, func, retPointer);
 		break;
 
 	case ICC_VIRTUAL_THISCALL:
+	case ICC_VIRTUAL_THISCALL_OBJFIRST:
+	case ICC_VIRTUAL_THISCALL_OBJLAST:
 		{
 			// Get virtual function table from the object pointer
 			asFUNCTION_t *vftable = *(asFUNCTION_t**)obj;
@@ -192,6 +225,8 @@ asQWORD CallSystemFunctionNative(asCContext *context, asCScriptFunction *descr,
 		break;
 
 	case ICC_VIRTUAL_THISCALL_RETURNINMEM:
+	case ICC_VIRTUAL_THISCALL_OBJFIRST_RETURNINMEM:
+	case ICC_VIRTUAL_THISCALL_OBJLAST_RETURNINMEM:
 		{
 			// Get virtual function table from the object pointer
 			asFUNCTION_t *vftable = *(asFUNCTION_t**)obj;
