@@ -28,6 +28,7 @@
 #include <pthread.h>
 #include <stdexcept>
 #include <algorithm>
+#include <cerrno>
 #include <map>
 
 #include <stdio.h>
@@ -206,6 +207,22 @@ void SFXManager::queue(SFXCommands command, SFXBase *sfx, const Vec3 &p)
    queueCommand(sfx_command);
 }   // queue (Vec3)
 
+//----------------------------------------------------------------------------
+/** Adds a sound effect command with a float and a Vec3 parameter to the queue
+ *  of the sfx manager. Openal commands can sometimes cause a 5ms delay, so it
+ *   is done in a separate thread.
+ *  \param command The command to execute.
+ *  \param sfx The sound effect to be started.
+ *  \param f A float parameter for the command.
+ *  \param p A Vec3 parameter for the command.
+ */
+void SFXManager::queue(SFXCommands command, SFXBase *sfx, float f,
+                       const Vec3 &p)
+{
+    SFXCommand *sfx_command = new SFXCommand(command, sfx, f, p);
+    queueCommand(sfx_command);
+}   // queue(float, Vec3)
+
 //----------------------------------------------------------------------------
 /** Queues a command for the music manager.
  *  \param mi The music for which the command is.
@@ -240,7 +257,8 @@ void SFXManager::queueCommand(SFXCommand *command)
         race_manager->getMinorMode() != RaceManager::MINOR_MODE_CUTSCENE)
     {
         if(command->m_command==SFX_POSITION || command->m_command==SFX_LOOP ||
-           command->m_command==SFX_SPEED                                       )
+           command->m_command==SFX_SPEED    || 
+           command->m_command==SFX_SPEED_POSITION                               )
         {
             delete command;
             static int count_messages = 0;
@@ -309,20 +327,26 @@ void* SFXManager::mainLoop(void *obj)
         switch (current->m_command)
         {
         case SFX_PLAY:     current->m_sfx->reallyPlayNow();       break;
+        case SFX_PLAY_POSITION:
+            current->m_sfx->reallyPlayNow(current->m_parameter);  break;
         case SFX_STOP:     current->m_sfx->reallyStopNow();       break;
         case SFX_PAUSE:    current->m_sfx->reallyPauseNow();      break;
         case SFX_RESUME:   current->m_sfx->reallyResumeNow();     break;
         case SFX_SPEED:    current->m_sfx->reallySetSpeed(
-            current->m_parameter.getX());   break;
+                                  current->m_parameter.getX());   break;
         case SFX_POSITION: current->m_sfx->reallySetPosition(
-            current->m_parameter);   break;
+                                         current->m_parameter);   break;
+        case SFX_SPEED_POSITION: current->m_sfx->reallySetSpeedPosition(
+                                         // Extract float from W component
+                                         current->m_parameter.getW(),
+                                         current->m_parameter);   break;
         case SFX_VOLUME:   current->m_sfx->reallySetVolume(
-            current->m_parameter.getX());   break;
+                                  current->m_parameter.getX());   break;
         case SFX_MASTER_VOLUME:
             current->m_sfx->reallySetMasterVolumeNow(
-                current->m_parameter.getX());   break;
+                                  current->m_parameter.getX());   break;
         case SFX_LOOP:     current->m_sfx->reallySetLoop(
-            current->m_parameter.getX() != 0);   break;
+                             current->m_parameter.getX() != 0);   break;
         case SFX_DELETE:     me->deleteSFX(current->m_sfx);       break;
         case SFX_PAUSE_ALL:  me->reallyPauseAllNow();             break;
         case SFX_RESUME_ALL: me->reallyResumeAllNow();            break;
@@ -938,7 +962,7 @@ SFXBase* SFXManager::quickSound(const std::string &sound_type)
 {
     if (!sfxAllowed()) return NULL;
 
-    m_quick_sounds.lock();
+    MutexLockerHelper lock(m_quick_sounds);
     std::map<std::string, SFXBase*>::iterator sound = 
                                      m_quick_sounds.getData().find(sound_type);
 
@@ -949,14 +973,13 @@ SFXBase* SFXManager::quickSound(const std::string &sound_type)
         if (new_sound == NULL) return NULL;
         new_sound->play();
         m_quick_sounds.getData()[sound_type] = new_sound;
-        m_quick_sounds.unlock();
         return new_sound;
     }
     else
     {
         SFXBase *base_sound = sound->second;
-        base_sound->play();
-        m_quick_sounds.unlock();
+        if (base_sound->getStatus() != SFXBase::SFX_PLAYING)
+            base_sound->play();
         return base_sound;
     }
 
