@@ -513,7 +513,8 @@ namespace UserConfigParams
     PARAM_PREFIX bool m_check_debug PARAM_DEFAULT( false );
 
     /** Special debug camera: 0: normal camera;   1: being high over the kart;
-                              2: on ground level; 3: free first person camera; */
+                              2: on ground level; 3: free first person camera; 
+                              4: straight behind kart */
     PARAM_PREFIX int m_camera_debug PARAM_DEFAULT( false );
 
     /** True if physics debugging should be enabled. */
