@@ -39,7 +39,7 @@ namespace GraphicsRestrictions
         /** The list of names used in the XML file for the graphics
          *  restriction types. They must be in the same order as the types. */
 
-        char *m_names_of_restrictions[] = {
+        const char *m_names_of_restrictions[] = {
             "UniformBufferObject",
             "GeometryShader4",
             "DrawIndirect",
