@@ -262,6 +262,7 @@ void RibbonWidget::add()
                 label->setTextAlignment(EGUIA_CENTER, EGUIA_CENTER);
                 label->setTabStop(false);
                 label->setNotClipped(true);
+                label->setRightToLeft(translations->isRTLText(message));
                 m_labels.push_back(label);
 
                 subbtn->setTabStop(false);
@@ -559,7 +560,9 @@ EventPropagation RibbonWidget::focused(const int playerID)
     {
         if (m_selection[playerID] != -1)
         {
-            m_active_children.get(m_selection[playerID])->focused(playerID);
+            int selection = m_selection[playerID];
+            if (selection < (int)m_active_children.size())
+                m_active_children.get(selection)->focused(playerID);
         }
     }
 
