@@ -40,6 +40,7 @@
 #include "race/race_manager.hpp"
 #include "scriptengine/script_engine.hpp"
 #include "tracks/track.hpp"
+#include "tracks/track_object.hpp"
 #include "utils/profiler.hpp"
 
 // ----------------------------------------------------------------------------
@@ -171,9 +172,11 @@ void Physics::update(float dt)
             Scripting::ScriptEngine* script_engine = World::getWorld()->getScriptEngine();
             int kartid1 = p->getUserPointer(0)->getPointerKart()->getWorldKartId();
             int kartid2 = p->getUserPointer(1)->getPointerKart()->getWorldKartId();
-            Scripting::Physics::setCollision(kartid1,kartid2);
-            Scripting::Physics::setCollisionType("KartKart");
-            script_engine->runScript("collisions");
+            script_engine->runFunction(false, "void onKartKartCollision(int, int)",
+                [=](asIScriptContext* ctx) {
+                    ctx->SetArgDWord(0, kartid1);
+                    ctx->SetArgDWord(1, kartid2);
+                });
             continue;
         }  // if kart-kart collision
 
@@ -182,29 +185,39 @@ void Physics::update(float dt)
             // Kart hits physical object
             // -------------------------
             Scripting::ScriptEngine* script_engine = World::getWorld()->getScriptEngine();
-            Scripting::Physics::setCollision(0, 0);
-            Scripting::Physics::setCollisionType("KartObject"); //object as in physical object
-            Scripting::Physics::setCollision
-                (
-                p->getUserPointer(0)->getPointerPhysicalObject()->getID(),
-                "kart"
-                );
-            script_engine->runScript("collisions");
-            PhysicalObject *obj = p->getUserPointer(0)
-                                   ->getPointerPhysicalObject();
-            if(obj->isCrashReset())
+            AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
+            int kartId = kart->getWorldKartId();
+            PhysicalObject* obj = p->getUserPointer(0)->getPointerPhysicalObject();
+            std::string obj_id = obj->getID();
+            std::string scripting_function = obj->getOnKartCollisionFunction();
+
+            TrackObject* to = obj->getTrackObject();
+            TrackObject* library = to->getParentLibrary();
+            std::string lib_id;
+            std::string* lib_id_ptr = NULL;
+            if (library != NULL)
+                lib_id = library->getID();
+            lib_id_ptr = &lib_id;
+
+            if (scripting_function.size() > 0)
+            {
+                script_engine->runFunction(true, "void " + scripting_function + "(int, const string, const string)",
+                    [&](asIScriptContext* ctx) {
+                        ctx->SetArgDWord(0, kartId);
+                        ctx->SetArgObject(1, lib_id_ptr);
+                        ctx->SetArgObject(2, &obj_id);
+                    });
+            }
+            if (obj->isCrashReset())
             {
-                AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
                 new RescueAnimation(kart);
             }
             else if (obj->isExplodeKartObject())
             {
-                AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
                 ExplosionAnimation::create(kart);
             }
             else if (obj->isFlattenKartObject())
             {
-                AbstractKart *kart = p->getUserPointer(1)->getPointerKart();
                 const KartProperties* kp = kart->getKartProperties();
                 kart->setSquash(kp->getSquashDuration() * kart->getPlayerDifficulty()->getSquashDuration(),
                     kp->getSquashSlowdown() * kart->getPlayerDifficulty()->getSquashSlowdown());
@@ -212,14 +225,13 @@ void Physics::update(float dt)
             else if(obj->isSoccerBall() && 
                     race_manager->getMinorMode() == RaceManager::MINOR_MODE_SOCCER)
             {
-                int kartId = p->getUserPointer(1)->getPointerKart()->getWorldKartId();
                 SoccerWorld* soccerWorld = (SoccerWorld*)World::getWorld();
                 soccerWorld->setLastKartTohitBall(kartId);
             }
             continue;
         }
 
-        if(p->getUserPointer(0)->is(UserPointer::UP_ANIMATION))
+        if (p->getUserPointer(0)->is(UserPointer::UP_ANIMATION))
         {
             // Kart hits animation
             ThreeDAnimation *anim=p->getUserPointer(0)->getPointerAnimation();
@@ -256,17 +268,20 @@ void Physics::update(float dt)
             // Projectile hits physical object
             // -------------------------------
             Scripting::ScriptEngine* script_engine = World::getWorld()->getScriptEngine();
-            Scripting::Physics::setCollision(0,0); //TODO : support item types etc
-            Scripting::Physics::setCollisionType("ItemObject");
-            Scripting::Physics::setCollision
-                (
-                p->getUserPointer(1)->getPointerPhysicalObject()->getID(),
-                "item"
-                );
-            script_engine->runScript("collisions");
-            p->getUserPointer(0)->getPointerFlyable()
-                ->hit(NULL, p->getUserPointer(1)->getPointerPhysicalObject());
+            Flyable* flyable = p->getUserPointer(0)->getPointerFlyable();
             PhysicalObject* obj = p->getUserPointer(1)->getPointerPhysicalObject();
+            std::string obj_id = obj->getID();
+            std::string scripting_function = obj->getOnItemCollisionFunction();
+            if (scripting_function.size() > 0)
+            {
+                script_engine->runFunction(true, "void " + scripting_function + "(int, int, const string)",
+                        [&](asIScriptContext* ctx) {
+                        ctx->SetArgDWord(0, (int)flyable->getType());
+                        ctx->SetArgDWord(1, flyable->getOwnerId());
+                        ctx->SetArgObject(2, &obj_id);
+                    });
+            }
+            flyable->hit(NULL, obj);
 
             if (obj->isSoccerBall() && 
                 race_manager->getMinorMode() == RaceManager::MINOR_MODE_SOCCER)
@@ -301,8 +316,11 @@ void Physics::update(float dt)
                 if (target_kart != kart && c &&
                     c->getPlayer()->getConstProfile() == PlayerManager::getCurrentPlayer())
                 {
+                    // Compare the current value of hits with the 'hit' goal value
+                    // (otherwise it would be compared with the kart id goal value,
+                    // which doesn't exist.
                     PlayerManager::increaseAchievement(AchievementInfo::ACHIEVE_ARCH_ENEMY,
-                                                       target_kart->getIdent(), 1);
+                                                       target_kart->getIdent(), 1, "hit");
                     if (type == PowerupManager::POWERUP_BOWLING)
                     {
                         PlayerManager::increaseAchievement(AchievementInfo::ACHIEVE_STRIKE,
@@ -576,10 +594,21 @@ btScalar Physics::solveGroup(btCollisionObject** bodies, int numBodies,
                     upA, contact_manifold->getContactPoint(0).m_localPointA,
                     upB, contact_manifold->getContactPoint(0).m_localPointB);
             else if(upB->is(UserPointer::UP_PHYSICAL_OBJECT))
+            {
                 // 2.3 kart hits physical object
                 m_all_collisions.push_back(
                     upB, contact_manifold->getContactPoint(0).m_localPointB,
                     upA, contact_manifold->getContactPoint(0).m_localPointA);
+                // If the object is a statical object (e.g. a door in
+                // overworld) add a push back to avoid that karts get stuck
+                if (objB->isStaticObject())
+                {
+                    AbstractKart *kart = upA->getPointerKart();
+                    const btVector3 &normal = contact_manifold->getContactPoint(0)
+                        .m_normalWorldOnB;
+                    kart->crashed((Material*)NULL, normal);
+                }   // isStatiObject
+            }
             else if(upB->is(UserPointer::UP_ANIMATION))
                 m_all_collisions.push_back(
                     upB, contact_manifold->getContactPoint(0).m_localPointB,
