@@ -111,16 +111,9 @@ void SoccerSetupScreen::eventCallback(Widget* widget, const std::string& name,
     else if(name == "time_enabled")
     {
         CheckBoxWidget* timeEnabled = dynamic_cast<CheckBoxWidget*>(widget);
-        if(timeEnabled->getState())
-        {
-            getWidget<SpinnerWidget>("goalamount")->setDeactivated();
-            getWidget<SpinnerWidget>("timeamount")->setActivated();
-        }
-        else
-        {
-            getWidget<SpinnerWidget>("timeamount")->setDeactivated();
-            getWidget<SpinnerWidget>("goalamount")->setActivated();
-        }
+        bool timed = timeEnabled->getState();
+        getWidget<SpinnerWidget>("goalamount")->setActive(!timed);
+        getWidget<SpinnerWidget>("timeamount")->setActive(timed);
     }
 }   // eventCallback
 
@@ -208,11 +201,11 @@ void SoccerSetupScreen::init()
 
     SpinnerWidget*  goalamount = getWidget<SpinnerWidget>("goalamount");
     goalamount->setValue(UserConfigParams::m_num_goals);
-    goalamount->setActivated();
+    goalamount->setActive(true);
 
     SpinnerWidget* timeAmount = getWidget<SpinnerWidget>("timeamount");
     timeAmount->setValue(UserConfigParams::m_soccer_time_limit);
-    timeAmount->setDeactivated();
+    timeAmount->setActive(false);
 
     CheckBoxWidget* timeEnabled = getWidget<CheckBoxWidget>("time_enabled");
     timeEnabled->setState(false);
