@@ -28,7 +28,7 @@ public:
   StkFileSystem();
 
   std::vector<std::string>    open_directory(const std::string& pathname);
-  std::auto_ptr<std::istream> open_file(const std::string& filename);
+  std::unique_ptr<std::istream> open_file(const std::string& filename);
 };
 
 } // namespace tinygettext
