@@ -17,27 +17,32 @@
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "tracks/track_object_presentation.hpp"
-#include "graphics/central_settings.hpp"
+
 #include "audio/sfx_base.hpp"
 #include "audio/sfx_buffer.hpp"
 #include "challenges/unlock_manager.hpp"
 #include "config/user_config.hpp"
+#include "graphics/camera.hpp"
+#include "graphics/central_settings.hpp"
 #include "graphics/irr_driver.hpp"
 #include "graphics/material_manager.hpp"
 #include "graphics/mesh_tools.hpp"
 #include "graphics/particle_emitter.hpp"
 #include "graphics/particle_kind_manager.hpp"
-#include "graphics/stkmeshscenenode.hpp"
+#include "graphics/stk_mesh_scene_node.hpp"
 #include "io/file_manager.hpp"
 #include "io/xml_node.hpp"
 #include "input/device_manager.hpp"
 #include "input/input_device.hpp"
 #include "input/input_manager.hpp"
 #include "items/item_manager.hpp"
+#include "karts/abstract_kart.hpp"
 #include "modes/world.hpp"
 #include "scriptengine/script_engine.hpp"
-#include "states_screens/dialogs/race_paused_dialog.hpp"
 #include "states_screens/dialogs/tutorial_message_dialog.hpp"
+#include "tracks/check_cylinder.hpp"
+#include "tracks/check_manager.hpp"
+#include "tracks/check_sphere.hpp"
 #include "tracks/model_definition_loader.hpp"
 #include "tracks/track.hpp"
 #include "tracks/track_manager.hpp"
@@ -83,6 +88,16 @@ const core::vector3df TrackObjectPresentationSceneNode::getAbsolutePosition() co
     return m_node->getAbsolutePosition();
 }   // getAbsolutePosition
 
+// ----------------------------------------------------------------------------
+
+const core::vector3df TrackObjectPresentationSceneNode::getAbsoluteCenterPosition() const
+{
+    if (m_node == NULL) return m_init_xyz;
+    m_node->updateAbsolutePosition();
+    core::aabbox3d<f32> bounds = m_node->getTransformedBoundingBox();
+    return bounds.getCenter();
+}
+
 // ----------------------------------------------------------------------------
 const core::vector3df& TrackObjectPresentationSceneNode::getRotation() const
 {
@@ -123,7 +138,7 @@ void TrackObjectPresentationSceneNode::move(const core::vector3df& xyz,
 // ----------------------------------------------------------------------------
 void TrackObjectPresentationSceneNode::setEnable(bool enabled)
 {
-    if (m_node != NULL)
+    if (m_node != NULL && (!enabled || !m_force_always_hidden))
         m_node->setVisible(enabled);
 }   // setEnable
 
@@ -155,9 +170,10 @@ TrackObjectPresentationEmpty::~TrackObjectPresentationEmpty()
 
 // ----------------------------------------------------------------------------
 TrackObjectPresentationLibraryNode::TrackObjectPresentationLibraryNode(
-                                       const XMLNode& xml_node,
-                                       ModelDefinitionLoader& model_def_loader)
-                                  : TrackObjectPresentationSceneNode(xml_node)
+    TrackObject* parent,
+    const XMLNode& xml_node,
+    ModelDefinitionLoader& model_def_loader)
+    : TrackObjectPresentationSceneNode(xml_node)
 {
     std::string name;
     xml_node.get("name", &name);
@@ -180,18 +196,27 @@ TrackObjectPresentationLibraryNode::TrackObjectPresentationLibraryNode(
         if (world != NULL)
             track = world->getTrack();
         std::string local_lib_node_path;
+        std::string local_script_file_path;
         if (track != NULL)
+        {
             local_lib_node_path = track->getTrackFile("library/" + name + "/node.xml");
+            local_script_file_path = track->getTrackFile("library/" + name + "/scripting.as");
+        }
         std::string lib_node_path = lib_path + "node.xml";
+        std::string lib_script_file_path = lib_path + "scripting.as";
 
         if (local_lib_node_path.size() > 0 && file_manager->fileExists(local_lib_node_path))
         {
             lib_path = track->getTrackFile("library/" + name);
             libroot = file_manager->createXMLTree(local_lib_node_path);
+            if (track != NULL)
+                World::getWorld()->getScriptEngine()->loadScript(local_script_file_path, false);
         }
         else if (file_manager->fileExists(lib_node_path))
         {
             libroot = file_manager->createXMLTree(lib_node_path);
+            if (track != NULL)
+                World::getWorld()->getScriptEngine()->loadScript(lib_script_file_path, false);
         }
         else
         {
@@ -241,7 +266,8 @@ TrackObjectPresentationLibraryNode::TrackObjectPresentationLibraryNode(
 
     assert(libroot != NULL);
     World::getWorld()->getTrack()->loadObjects(libroot, lib_path, model_def_loader,
-        create_lod_definitions, m_node);
+        create_lod_definitions, m_node, parent);
+    m_parent = parent;
 }   // TrackObjectPresentationLibraryNode
 
 // ----------------------------------------------------------------------------
@@ -249,7 +275,24 @@ TrackObjectPresentationLibraryNode::~TrackObjectPresentationLibraryNode()
 {
     irr_driver->removeNode(m_node);
 }   // TrackObjectPresentationLibraryNode
+// ----------------------------------------------------------------------------
+void TrackObjectPresentationLibraryNode::move(const core::vector3df& xyz, const core::vector3df& hpr,
+    const core::vector3df& scale, bool isAbsoluteCoord, bool moveChildrenPhysicalBodies)
+{
+    TrackObjectPresentationSceneNode::move(xyz, hpr, scale, isAbsoluteCoord);
 
+    if (moveChildrenPhysicalBodies)
+    {
+        for (TrackObject* obj : m_parent->getChildren())
+        {
+            obj->reset();
+            if (obj->getPhysicalObject() != NULL)
+            {
+                obj->movePhysicalBodyToGraphicalNode(obj->getAbsolutePosition(), obj->getRotation());
+            }
+        }
+    }
+}
 // ----------------------------------------------------------------------------
 TrackObjectPresentationLOD::TrackObjectPresentationLOD(const XMLNode& xml_node,
                                        scene::ISceneNode* parent,
@@ -397,13 +440,30 @@ void TrackObjectPresentationMesh::init(const XMLNode* xml_node,
 
     if (interaction == "physicsonly")
     {
-        m_node = irr_driver->addMesh(m_mesh, m_model_file, parent);
-        enabled = false;
-        m_frame_start = 0;
-        m_frame_end = 0;
+        std::string type;
+        xml_node->get("type", &type);
+        if (type == "animation" || xml_node->hasChildNamed("curve"))
+        {
+            // Animated
+            //m_node = irr_driver->getSceneManager()->addEmptySceneNode();
+            m_node = irr_driver->addMesh(m_mesh, m_model_file, parent);
+            enabled = false;
+            m_force_always_hidden = true;
+            m_frame_start = 0;
+            m_frame_end = 0;
+        }
+        else
+        {
+            // Static
+            m_node = irr_driver->addMesh(m_mesh, m_model_file, parent);
+            enabled = false;
+            m_force_always_hidden = true;
+            m_frame_start = 0;
+            m_frame_end = 0;
 
-        if (World::getWorld() && World::getWorld()->getTrack() && xml_node)
-            World::getWorld()->getTrack()->addPhysicsOnlyNode(m_node);
+            if (World::getWorld() && World::getWorld()->getTrack() && xml_node)
+                World::getWorld()->getTrack()->addPhysicsOnlyNode(m_node);
+        }
     }
     else if (m_is_in_skybox)
     {
@@ -550,6 +610,7 @@ TrackObjectPresentationSound::TrackObjectPresentationSound(
 {
     // TODO: respect 'parent' if any
 
+    m_enabled = true;
     m_sound = NULL;
     m_xyz   = m_init_xyz;
 
@@ -609,7 +670,7 @@ TrackObjectPresentationSound::TrackObjectPresentationSound(
 // ----------------------------------------------------------------------------
 void TrackObjectPresentationSound::update(float dt)
 {
-    if (m_sound != NULL)
+    if (m_sound != NULL && m_enabled)
     {
         // muting when too far is implemented manually since not supported by
         // OpenAL so need to call this every frame to update the muting state
@@ -619,9 +680,9 @@ void TrackObjectPresentationSound::update(float dt)
 }   // update
 
 // ----------------------------------------------------------------------------
-void TrackObjectPresentationSound::onTriggerItemApproached(Item* who)
+void TrackObjectPresentationSound::onTriggerItemApproached()
 {
-    if (m_sound != NULL && m_sound->getStatus() != SFXBase::SFX_PLAYING)
+    if (m_sound != NULL && m_sound->getStatus() != SFXBase::SFX_PLAYING && m_enabled)
     {
         m_sound->play();
     }
@@ -630,7 +691,7 @@ void TrackObjectPresentationSound::onTriggerItemApproached(Item* who)
 // ----------------------------------------------------------------------------
 void TrackObjectPresentationSound::triggerSound(bool loop)
 {
-    if (m_sound != NULL)
+    if (m_sound != NULL && m_enabled)
     {
         m_sound->setLoop(loop);
         m_sound->play();
@@ -640,7 +701,8 @@ void TrackObjectPresentationSound::triggerSound(bool loop)
 // ----------------------------------------------------------------------------
 void TrackObjectPresentationSound::stopSound()
 {
-    if (m_sound != NULL) m_sound->stop();
+    if (m_sound != NULL) 
+        m_sound->stop();
 }   // stopSound
 
 // ----------------------------------------------------------------------------
@@ -659,9 +721,24 @@ void TrackObjectPresentationSound::move(const core::vector3df& xyz,
                                         bool isAbsoluteCoord)
 {
     m_xyz = xyz;
-    if (m_sound != NULL) m_sound->setPosition(xyz);
+    if (m_sound != NULL && m_enabled)
+        m_sound->setPosition(xyz);
 }   // move
 
+// ----------------------------------------------------------------------------
+
+void TrackObjectPresentationSound::setEnable(bool enabled)
+{
+    if (enabled != m_enabled)
+    {
+        m_enabled = enabled;
+        if (enabled)
+            triggerSound(true);
+        else
+            stopSound();
+    }
+}
+
 // ----------------------------------------------------------------------------
 TrackObjectPresentationBillboard::TrackObjectPresentationBillboard(
                                                      const XMLNode& xml_node, 
@@ -754,6 +831,12 @@ TrackObjectPresentationParticles::TrackObjectPresentationParticles(
     xml_node.get("clip_distance", &clip_distance);
     xml_node.get("conditions",    &m_trigger_condition);
 
+    bool auto_emit = true;
+    xml_node.get("auto_emit", &auto_emit);
+
+    m_delayed_stop = false;
+    m_delayed_stop_time = 0.0;
+
     try
     {
         ParticleKind* kind = ParticleKindManager::get()->getParticles(path);
@@ -780,7 +863,7 @@ TrackObjectPresentationParticles::TrackObjectPresentationParticles(
             m_emitter = emitter;
         }
 
-        if (m_trigger_condition.size() > 0)
+        if (m_trigger_condition.size() > 0 || !auto_emit)
         {
             m_emitter->setCreationRateAbsolute(0.0f);
         }
@@ -813,6 +896,16 @@ void TrackObjectPresentationParticles::update(float dt)
     {
         m_emitter->update(dt);
     }
+
+    if (m_delayed_stop)
+    {
+        if (m_delayed_stop_time < 0.0f)
+        {
+            m_delayed_stop = false;
+            stop();
+        }
+        m_delayed_stop_time -= dt;
+    }
 }   // update
 
 // ----------------------------------------------------------------------------
@@ -824,7 +917,30 @@ void TrackObjectPresentationParticles::triggerParticles()
         m_emitter->setParticleType(m_emitter->getParticlesInfo());
     }
 }   // triggerParticles
-
+// ----------------------------------------------------------------------------
+void TrackObjectPresentationParticles::stop()
+{
+    if (m_emitter != NULL)
+    {
+        m_emitter->setCreationRateAbsolute(0.0f);
+        m_emitter->clearParticles();
+    }
+}
+// ----------------------------------------------------------------------------
+void TrackObjectPresentationParticles::stopIn(double delay)
+{
+    m_delayed_stop = true;
+    m_delayed_stop_time = delay;
+}
+// ----------------------------------------------------------------------------
+void TrackObjectPresentationParticles::setRate(float rate)
+{
+    if (m_emitter != NULL)
+    {
+        m_emitter->setCreationRateAbsolute(rate);
+        m_emitter->setParticleType(m_emitter->getParticlesInfo());
+    }
+}
 // ----------------------------------------------------------------------------
 TrackObjectPresentationLight::TrackObjectPresentationLight(
                                                      const XMLNode& xml_node, 
@@ -866,12 +982,40 @@ TrackObjectPresentationActionTrigger::TrackObjectPresentationActionTrigger(
     xml_node.get("distance", &trigger_distance);
     xml_node.get("action",   &m_action        );
 
+    std::string trigger_type;
+    xml_node.get("trigger-type", &trigger_type);
+    if (trigger_type == "point" || trigger_type.empty())
+    {
+        m_type = TRIGGER_TYPE_POINT;
+    }
+    else if (trigger_type == "cylinder")
+    {
+        m_type = TRIGGER_TYPE_CYLINDER;
+    }
+    else
+    {
+        assert(false);
+    }
+
     m_action_active = true;
 
     if (m_action.size() == 0)
         Log::warn("TrackObject", "Action-trigger has no action defined.");
 
-    ItemManager::get()->newItem(m_init_xyz, trigger_distance, this);
+    if (m_type == TRIGGER_TYPE_POINT)
+    {
+        // TODO: rewrite as a sphere check structure?
+        ItemManager::get()->newItem(m_init_xyz, trigger_distance, this);
+        // CheckManager::get()->add(new CheckSphere(xml_node, 0 /* TODO what is this? */));
+    }
+    else if (m_type == TRIGGER_TYPE_CYLINDER)
+    {
+        CheckManager::get()->add(new CheckCylinder(xml_node, 0 /* TODO what is this? */, this));
+    }
+    else
+    {
+        assert(false);
+    }
 }   // TrackObjectPresentationActionTrigger
 
 // ----------------------------------------------------------------------------
@@ -887,174 +1031,22 @@ TrackObjectPresentationActionTrigger::TrackObjectPresentationActionTrigger(
     float trigger_distance = distance;
     m_action               = script_name;
     m_action_active        = true;
+    m_type                 = TRIGGER_TYPE_POINT;
     ItemManager::get()->newItem(m_init_xyz, trigger_distance, this);
 }   // TrackObjectPresentationActionTrigger
 
 // ----------------------------------------------------------------------------
-void TrackObjectPresentationActionTrigger::onTriggerItemApproached(Item* who)
+void TrackObjectPresentationActionTrigger::onTriggerItemApproached()
 {
     if (!m_action_active) return;
 
-    // TODO: replace all of these hardcoded actions with scripting
-
-    if (m_action == "garage")
-    {
-        m_action_active = false;
-
-        new RacePausedDialog(0.8f, 0.6f);
-        //dynamic_cast<OverWorld*>(World::getWorld())->scheduleSelectKart();
-    }
-    //action trigger near big doors in the overword to notify players that
-    // they'll open once they finish all the challenges
-    else if (m_action == "big_door")
-    {
-        m_action_active = false;
-
-        Track* m_track = World::getWorld()->getTrack();
-        unsigned int unlocked_challenges = m_track->getNumOfCompletedChallenges();
-        std::vector<OverworldChallenge> m_challenges = m_track->getChallengeList();
-
-        // allow ONE unsolved challenge : the last one
-        if (unlocked_challenges < m_challenges.size() - 1)
-        {
-            new TutorialMessageDialog(
-                  _("Complete all challenges to unlock the big door!"), true);
-        }
-    }
-    else if (m_action == "tutorial_drive")
-    {
-        //if (World::getWorld()->getPhase() == World::RACE_PHASE)
-        {
-            m_action_active = false;
-            //World::getWorld()->getRaceGUI()->clearAllMessages();
-
-            InputDevice* device = input_manager->getDeviceManager()
-                                               ->getLatestUsedDevice();
-            DeviceConfig* config = device->getConfiguration();
-            irr::core::stringw accel = config->getBindingAsString(PA_ACCEL);
-            irr::core::stringw left = config->getBindingAsString(PA_STEER_LEFT);
-            irr::core::stringw right = config->getBindingAsString(PA_STEER_RIGHT);
-
-            new TutorialMessageDialog(_("Accelerate with <%s> and steer with "
-                                        "<%s> and <%s>", accel, left, right),
-                false);
-        }
-    }
-    else if (m_action == "tutorial_bananas")
-    {
-        m_action_active = false;
-
-        new TutorialMessageDialog(_("Avoid bananas!"), true);
-    }
-    else if (m_action == "tutorial_giftboxes")
-    {
-        m_action_active = false;
-        InputDevice* device = input_manager->getDeviceManager()
-                                           ->getLatestUsedDevice();
-        DeviceConfig* config = device->getConfiguration();
-        irr::core::stringw fire = config->getBindingAsString(PA_FIRE);
-
-        new TutorialMessageDialog(_("Collect gift boxes, and fire the weapon "
-                                    "with <%s> to blow away these boxes!",
-                                    fire),true);
-    }
-    else if (m_action == "tutorial_backgiftboxes")
-    {
-        m_action_active = false;
-        InputDevice* device = input_manager->getDeviceManager()
-                                           ->getLatestUsedDevice();
-        DeviceConfig* config = device->getConfiguration();
-        irr::core::stringw fire = config->getBindingAsString(PA_FIRE);
-        irr::core::stringw back = config->getBindingAsString(PA_LOOK_BACK);
-
-    new TutorialMessageDialog(
-           _("Press <%s> to look behind, and fire the weapon with <%s> while "
-             "pressing <%s> to fire behind!", back, fire, back),
-            true);
-    }
-    else if (m_action == "tutorial_nitro_collect")
-    {
-        m_action_active = false;
-
-        new TutorialMessageDialog(_("Collect nitro bottles (we will use them "
-                                    "after the curve)"), true);
-    }
-    else if (m_action == "tutorial_nitro_use")
-    {
-        m_action_active = false;
-        InputDevice* device = input_manager->getDeviceManager()
-                                           ->getLatestUsedDevice();
-        DeviceConfig* config = device->getConfiguration();
-        irr::core::stringw nitro = config->getBindingAsString(PA_NITRO);
-
-        new TutorialMessageDialog(_("Use the nitro you collected by "
-                                    "pressing <%s>!", nitro), true);
-    }
-    else if (m_action == "tutorial_rescue")
-    {
-        m_action_active = false;
-        InputDevice* device = input_manager->getDeviceManager()
-                                           ->getLatestUsedDevice();
-        DeviceConfig* config = device->getConfiguration();
-        irr::core::stringw rescue = config->getBindingAsString(PA_RESCUE);
-
-        new TutorialMessageDialog(_("Oops! When you're in trouble, press <%s> "
-                                    "to be rescued", rescue),
-                                  false);
-    }
-    else if (m_action == "tutorial_skidding")
-    {
-        m_action_active = false;
-        //World::getWorld()->getRaceGUI()->clearAllMessages();
-
-        InputDevice* device = input_manager->getDeviceManager()
-                                           ->getLatestUsedDevice();
-        DeviceConfig* config = device->getConfiguration();
-        irr::core::stringw skid = config->getBindingAsString(PA_DRIFT);
-
-
-        new TutorialMessageDialog(_("Accelerate and press the <%s> key while "
-                                    "turning to skid. Skidding for a short "
-                                    "while can help you turn faster to take "
-                                    "sharp turns.", skid),
-            true);
-    }
-    else if (m_action == "tutorial_skidding2")
-    {
-        m_action_active = false;
-        World::getWorld()->getRaceGUI()->clearAllMessages();
-
-        new TutorialMessageDialog(_("Note that if you manage to skid for "
-                                    "several seconds, you will receive a bonus "
-                                    "speedup as a reward!"),
-            true);
-    }
-    else if (m_action == "tutorial_endmessage")
-    {
-        m_action_active = false;
-        World::getWorld()->getRaceGUI()->clearAllMessages();
-
-        new TutorialMessageDialog(_("You are now ready to race. Good luck!"),
-            true);
-    }
-    else if (m_action == "tutorial_exit")
-    {
-        World::getWorld()->scheduleExitRace();
-        return;
-    }
-    else
-    {	
-        //TODO move all above functions into scripts and remove the ifs
-        Scripting::ScriptEngine* m_script_engine =
-            World::getWorld()->getScriptEngine();
-        m_action_active = false;
-        m_script_engine->runScript(m_action);
-        
-        /*
-        Catch exception -> script not found
-        fprintf(stderr, "[TrackObject] WARNING: unknown action <%s>\n",
-                m_action.c_str());
-        
-         */
-    }
+    Scripting::ScriptEngine* script_engine =
+        World::getWorld()->getScriptEngine();
+    m_action_active = false; // TODO: allow auto re-activating?
+    int idKart = 0;
+    Camera* camera = Camera::getActiveCamera();
+    if (camera != NULL && camera->getKart() != NULL)
+        idKart = camera->getKart()->getWorldKartId();
+    script_engine->runFunction(true, "void " + m_action + "(int)",
+        [=](asIScriptContext* ctx) { ctx->SetArgDWord(0, idKart); });
 }   // onTriggerItemApproached
