@@ -22,15 +22,16 @@
 
 #include "utils/log.hpp"
 #include "utils/time.hpp"
-
+#include "utils/utf8.h"
 #include "coreutil.h"
 
-#include <math.h>
 #include <algorithm>
+#include <cassert>
+#include <cmath>
+#include <cstdio>
 #include <cstring>
-#include <stdio.h>
+#include <cwchar>
 #include <exception>
-#include <assert.h>
 
 namespace StringUtils
 {
@@ -167,7 +168,7 @@ namespace StringUtils
         try
         {
             std::string::size_type start=0;
-            while(start!=std::string::npos && start<(unsigned int)s.size())
+            while(start < (unsigned int) s.size())
             {
                 std::string::size_type i=s.find(c, start);
                 if (i!=std::string::npos)
@@ -185,11 +186,11 @@ namespace StringUtils
                 }
                 else   // end of string reached
                 {
-                    if (keepSplitChar)
+                    if (keepSplitChar && start != 0)
                         result.push_back(std::string(s,start-1));
                     else
                         result.push_back(std::string(s,start));
-                    start = i;
+                    return result;
                 }
             }
             return result;
@@ -243,14 +244,13 @@ namespace StringUtils
                 }
                 else
                 {
-                    if (keepSplitChar)
+                    if (keepSplitChar && start != 0)
                         result.push_back( s.subString(start - 1,
                                                       s.size()-start + 1) );
                     else
                         result.push_back( s.subString(start, s.size()-start) );
 
                     return result;
-                    //start = i+1;
                 }
             }
             return result;
@@ -684,6 +684,32 @@ namespace StringUtils
         return output.str();
     }   // xmlEncode
 
+    // ------------------------------------------------------------------------
+
+    std::string wide_to_utf8(const wchar_t* input)
+    {
+        static std::vector<char> utf8line;
+        utf8line.clear();
+
+        utf8::utf16to8(input, input + wcslen(input), back_inserter(utf8line));
+        utf8line.push_back(0);
+
+        return std::string(&utf8line[0]);
+    }
+
+    // ------------------------------------------------------------------------
+
+    irr::core::stringw utf8_to_wide(const char* input)
+    {
+        static std::vector<wchar_t> utf16line;
+        utf16line.clear();
+
+        utf8::utf8to16(input, input + strlen(input), back_inserter(utf16line));
+        utf16line.push_back(0);
+
+        return irr::core::stringw(&utf16line[0]);
+    }
+
     // ------------------------------------------------------------------------
     /** Converts a version string (in the form of 'X.Y.Za-rcU' into an
      *  integer number.
